// ---------------------------------------------------------------------------
// GeneSeqTester.cpp
// 
// Written by:   
// Last Updated:
//
// Description
// Program designed to test and grade the functionality of 
// C++ classes named CodeSeq and GeneSeq as declared in 
// CodeSeq.h and GeneSeq.h
// and defined/implemented in CodeSeq.cpp GeneSeq.cpp
//
// This is an EXAMPLE testing program.
// It or something very similar to it will be used to grade your code.
// In concept, if your code scores all the points using this tester, 
// then your code should score most (all) of the points on the actual
// testing program.
//
// The official testing program may 
// perform additional and/or different tests of similar design
// with different point weighting as appropriate.
//
// ---------------------------------------------------------------------------
#include <iostream>
#include <string>
#include "CodeSeq.h"
#include "GeneSeq.h"

// ---------------------------------------------------------------------------
// ProtoTypes - Global Function Declarations
// ---------------------------------------------------------------------------
int TestCodeSeq();
int TestCodeSeqExtras();
int TestGeneSeq();


using namespace std;

// ---------------------------------------------------------------------------
// Main Function - it all begins here
// ---------------------------------------------------------------------------
int main()
{
    int totalPoints = 0;
    
    cout << "In the main method of GeneSeqTester.cpp" << endl;
   
    // Test the fundamentals of base class CodeSeq 
    totalPoints += TestCodeSeq();

    // Test the additions to base class CodeSeq
    totalPoints += TestCodeSeqExtras();

    // Test the additions to base class CodeSeq
    totalPoints += TestGeneSeq();

    cout << "=================================================================\n";
    cout << "Automated Test Score = " << totalPoints << endl;    
   
    return 0;  // 0 indicates program success to the operating system
}

// ---------------------------------------------------------------------------
// TestCodeSeq
// Runs various function calls to test the behavior and functionality
// of the CodeSeq class.
//
// This tests the basic functionality of CodeSeq
//
// Returns total points earned (if it runs to completion)
// ---------------------------------------------------------------------------
int TestCodeSeq()
{
   string testStr = "catyy";
   int points     = 0;

   cout << endl << endl;
   cout << "Begin TestCodeSeq() -------------------------------------\n\n";

   // Test the constructor
   CodeSeq cs(testStr);	
   points += 1;
   cout << "Constructor works +1 point. Total: " << points << endl;


   // Test getLength() method
   if (cs.getLength() == 5)
   {
      points += 1;
      cout << "getLength() works +1 point. Total: " << points << endl;
   }
   else
   {
      cout << "Error: getLength() did not work" << endl;
   }
   
   // Test the output operator <<
   cout << "Outputting CodeSeq cs using operator<< "<< endl;
   cout << cs << endl << endl;
   points += 1;
   cout << "operator<< works +1 points. Total: " << points << endl;
   
   // Test getCodeSeq()
   string returnedStr = cs.getCodeSeq();

   if (returnedStr == testStr)
   {
      points += 1;
      cout << "getCodeSeq() works +1 points. Total: " << points << endl;
   }
   else
   {
      cout << "Error: getCodeSeq() did not return the sequence properly" << endl;
   }

   // Test CodeSeqs for equality
   // Case 1 is that they are equal
   CodeSeq code1("bbcca");
   CodeSeq code2("bbcca");
   
   if (code1 == code2)
   {
      points += 1;
      cout << "operator== works when codes are equal +1 point. Total: " << points << endl;
   }	
   else
   {
      cout << "Error: operator== failed when codes are equal" << endl;
   }
   // Can operator determine that they are not equal
   CodeSeq code3("abbcag");
   CodeSeq code4("abcagg");
   if (code3 == code4)
   {	
      cout << "Error: operator== failed when codes are NOT equal"<<endl;
   }
   else
   {
      points += 1;
      cout << "operator== works when codes are NOT equal +1 point. Total: " << points << endl;
   }

   // Testing of operator< for subsequence finding
   string fullStr = "actnowbeforeitstoolate";
   string subs[5];
   subs[0] = "act";
   subs[1] = "z";
   subs[2] = "ACT";
   subs[3] = "o";
   subs[4] = "atek";

   CodeSeq full(fullStr);
   CodeSeq sub1(subs[0]);
   CodeSeq sub2(subs[1]);
   CodeSeq sub3(subs[2]);
   CodeSeq sub4(subs[3]);
   CodeSeq sub5(subs[4]);

   for (int i=0; i < 5; i++)
   {
      cout << "Case " << i << ": " << subs[i] << " is a subsequence of " << fullStr <<endl;
      bool testPassed;
      testPassed = false;

      switch ( i )
      {
         case 0:
            testPassed = (sub1 < full);
            break;
         case 1: // to pass this test case, sub2 < full should return false
            testPassed = !(sub2 < full); 
            break;
         case 2: // to pass this test case, sub3 < full should return false
            testPassed = !(sub3 < full);
            break;
         case 3:
            testPassed = (sub4 < full);
            break;
         case 4: // to pass this test case, sub5 < full should return false
            testPassed = !(sub5 < full);
            break;
         default:
            cout << "Error in test program" << endl;
      }
      if (testPassed == true)
      {
         points += 1;
         cout << "operator< works for Case " << i << " gives +1 point. Total: " << points << endl;
      }
      else
      {
         cout<<"Failed operator< for Case " << i << endl;
      }
   } // end for i
   
   cout << "CodeSeq score is " << points << endl;
   cout << endl;

   return points;
}

// ---------------------------------------------------------------------------
// TestCodeSeqExtras
// Runs various function calls to test the behavior and functionality
// of the CodeSeq class.
//
// This tests the extended functionality of CodeSeq
//
// Returns total points earned (if it runs to completion)
// ---------------------------------------------------------------------------
int TestCodeSeqExtras()
{
   string testStr = "gatty123d_ccdq5gh";
   int points     = 0;

   cout << endl << endl;
   cout << "Begin TestCodeSeqExtras() -------------------------------------\n\n";

   // Test the copy constructor
   CodeSeq cs1(testStr);	
   CodeSeq cs2(cs1);	
   cout << "cs1 = [" << cs1 << "]\n";
   cout << "cs2 = [" << cs2 << "]\n";
   points += 5;
   cout << "Constructor works +5 points\n"; 
   cout << "                      2nd test Total: " << points << endl;
   
   // Test the assignment operator
   CodeSeq cs3("never_seen");	
   cs3 = cs2;
   cout << "cs2 = [" << cs2 << "]\n";
   cout << "cs3 = [" << cs3 << "]\n";
   points += 5;
   cout << "Assignment Op works +5 points\n"; 
   cout << "                      2nd test Total: " << points << endl;

   // Test the set member function of CodeSeq
   cs3.set(3, 'O');
   cout << "cs3[3] should be O\n";
   cout << "cs2 = [" << cs2 << "]\n";
   cout << "cs3 = [" << cs3 << "]\n";
   points += 3;
   cout << "set function check 1 works +3 points\n"; 
   cout << "                      2nd test Total: " << points << endl;
   
   // Test (check 2) the set member function of CodeSeq
   cs2.set(6, 'X');
   cout << "cs2[6] should be X\n";
   cout << "cs1 = [" << cs1 << "]\n";
   cout << "cs2 = [" << cs2 << "]\n";
   points += 2;
   cout << "set function check 2 works +2 points\n"; 
   cout << "                      2nd test Total: " << points << endl;
   
   // Test the resetSeq function
   cs1.resetSeq("blarg45");
   cout << "cs1 = [" << cs1 << "]\n";
   points += 5;
   cout << "resetSeq function works +5 points\n"; 
   cout << "                      2nd test Total: " << points << endl;

   return points;
}

// ---------------------------------------------------------------------------
// TestGeneSeq
// Runs various function calls to test the behavior and functionality
// of the GeneSeq class.
//
// Returns total points earned (if it runs to completion)
// ---------------------------------------------------------------------------
int TestGeneSeq()
{
   string testStr  = "daattcc345qaaggjkw";
   string testStr2 = "aattccaagg";
   int points     = 0;

   cout << endl << endl;
   cout << "Begin TestGeneSeq() -------------------------------------\n\n";

   // Test the copy constructor
   GeneSeq gs1(testStr);
   GeneSeq gs2(gs1);
   
   cout << "gs1 = [" << gs1 << "]\n";
   cout << "gs2 = [" << gs2 << "]\n";
   points += 5;
   cout << "Constructor works +5 points\n"; 
   cout << "                      3rd test Total: " << points << endl;
   
   // Test the assignment operator
   CodeSeq gs3("not_seen");	
   gs3 = gs2;
   cout << "gs2 = [" << gs2 << "]\n";
   cout << "gs3 = [" << gs3 << "]\n";
   points += 5;
   cout << "Assignment Op works +5 points\n"; 
   cout << "                      3rd test Total: " << points << endl;

   // Test comparison operator==
   CodeSeq gs4(testStr2);
   cout << "gs4 = [" << gs4 << "]\n";
   if (gs4 == gs1)
   {
	  points += 4;
      cout << "Comparison Op== works +4 points\n"; 
      cout << "                      3rd test Total: " << points << endl;
   }
   
   // Test resetSeq
   string testStr3 = "QWGGaaaattgghgcucucuag";
   string testStr4 = "aaaattgggcccag";
   GeneSeq gs5(testStr4);
   gs1.resetSeq(testStr3);
   cout << "gs1 = [" << gs1 << "]\n";
   if (gs5 == gs1)
   {
	  points += 5;
      cout << "resetSeq works +5 points\n"; 
      cout << "                      3rd test Total: " << points << endl;
   }
   
   // Test aCount
   if (gs5.aCount() == 5)
   {
	  points += 5;
      cout << "aCount works +5 points\n"; 
      cout << "                      3rd test Total: " << points << endl;
   }

   // Test cCount
   if (gs5.cCount() == 3)
   {
	  points += 5;
      cout << "cCount works +5 points\n"; 
      cout << "                      3rd test Total: " << points << endl;
   }
   
   // Test tCount
   if (gs5.tCount() == 2)
   {
	  points += 5;
      cout << "tCount works +5 points\n"; 
      cout << "                      3rd test Total: " << points << endl;
   }
   
   // Test gCount
   if (gs5.gCount() == 4)
   {
	  points += 5;
      cout << "gCount works +5 points\n"; 
      cout << "                      3rd test Total: " << points << endl;
   }
   
   return points;
}
